package de.fhswf;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.io.File;

import javax.swing.JComboBox;
import javax.swing.table.DefaultTableModel;

import de.fhswf.db.Testdb;

/**
 * 
 * Klasse mit Datenbankhilfsmethoden
 *
 * @author $Author:  Thomas Kramer
 * @version $Revision:  $, $Date:  $ UTC
 */
public class DataBaseHelper
{
   private static final String DataBaseFileName = "hsql_db";

   private static final File DataBaseFile = new File(DataBaseFileName
         + ".script");

   private static Testdb dataBase;

   /**
    * Default Constructor.
    */
   public DataBaseHelper()
   {
      initDataBase();
   }

   /**
    * Initialisiert eine HSQL-Datenbank. Beim erstmaligem Starten der
    * Datenbank wird eine neue Struktur angelegt. Andernfalls wird die
    * bestehende Struktur geladen.
    */
   private void initDataBase()
   {
      // Abfangen der mglichen SQL-Exceptions
      boolean DB_exists = DataBaseFile.isFile();
      try
      {
         // Starten der Datenbank
         dataBase = new Testdb(DataBaseFileName);

         // Prfen ob DataBaseFile noch nicht angelegt wurde
         if (!DB_exists)
         {
            // Anwendung wird mit eingerichteter Datenbank ausgeliefert.
         }
      }
      catch (Exception e)
      {
         e.printStackTrace();
         return;
      }
   }

   /**
    * 
    * Auslesen der Fahrer-Daten und bernehmen ins Datenmodell
    * 
    * @param dtm
    *           Datenmodell
    */
   public static void selectDrivers(DefaultTableModel dtm)
   {
      try
      {

         ResultSet rs = dataBase.query("SELECT * FROM personen");
         while (rs.next())
         {
            setNewDatas(
                  dtm,
                  new Object[] { rs.getString("name"),
                        rs.getString("fuehrerscheinklasse"),
                        rs.getString("fuehrerscheinseit") });
         }
      }
      catch (SQLException e)
      {
         e.printStackTrace();
      }
   }

   /**
    * 
    * Auslesen der Fahrzeug-Daten und bernehmen ins Datenmodell
    * 
    * @param dtm
    *           Datenmodell
    */
   public static void selectCars(DefaultTableModel dtm)
   {
      try
      {

         ResultSet rs = dataBase.query("SELECT * FROM fahrzeuge");
         while (rs.next())
         {
            setNewDatas(
                  dtm,
                  new Object[] { rs.getString("kennzeichen"),
                        rs.getString("erstzulassung") });
         }

      }
      catch (SQLException e)
      {
         e.printStackTrace();
      }
   }

   /**
    * 
    * Auslesen der Zuordnungs-Daten und bernehmen ins Datenmodell
    * 
    * @param dtm
    *           Datenmodell
    */
   public static void selectAssociations(DefaultTableModel dtm)
   {
      try
      {

         ResultSet rs = dataBase
               .query("SELECT * FROM zustaendigkeit");
         while (rs.next())
         {
            setNewDatas(
                  dtm,
                  new Object[] { rs.getString("fahrername"),
                        rs.getString("auto1"), rs.getString("auto2"),
                        rs.getString("auto3") });
         }

      }
      catch (SQLException e)
      {
         e.printStackTrace();
      }
   }
   
   /**
    * 
    * Fllen von Fahrzeugen (Kennzeichen) in bergebene Comboboxen
    *
    * @param car1 Auto1
    * @param car2 Auto2
    * @param car3 Auto3
    */
   public static void fillCarsComboboxes(JComboBox cars)
   {
      // zuerst leeren und leeren Eintrag hinzufgen      
      cars.removeAllItems();
      cars.addItem("");
      
      try
      {

         ResultSet rs = dataBase
               .query("SELECT * FROM FAHRZEUGE");
         while (rs.next())
         {
            cars.addItem(rs.getString("kennzeichen"));
         }

      }
      catch (SQLException e)
      {
         e.printStackTrace();
      }      
   }

   /**
    * 
    * Schreiben von Datenstzen ins Datenmodell
    *
    * @param dtm   Datenmodell
    * @param datas Daten
    */
   private static void setNewDatas(DefaultTableModel dtm,
         Object[] datas)
   {
      dtm.addRow(datas);
   }
   
   /**
    * 
    * Hinzufgen eines Fahrers
    *
    * @param name                  Name des Fahrers
    * @param fuehrerscheinklasse   Fhrerscheinklasse
    * @param fuehrerscheinSeit     FhrerscheinSeit
    */
   public static void insertNewDriver(String name, String fuehrerscheinklasse, String fuehrerscheinSeit)
   {
      update("Insert into personen values('" + name +"','" + fuehrerscheinklasse + "','" + fuehrerscheinSeit + "')");
   }
   
   /**
    * 
    * Hinzufgen einer Fahrer-Zuordnung
    *
    * @param fahrer Fahrer
    * @param auto1  Auto1
    * @param auto2  Auto2
    * @param auto3  Auto3
    */
   public static void insertNewAssociation(String fahrer, String auto1, String auto2, String auto3)
   {
      update("Insert into zustaendigkeit values('" + fahrer +"','" + auto1 + "','" + auto2 +  "','" + auto3 + "')");
   }      

   /**
    * Sendet einen SQL-Ausdruck ab. Bitte fr CREATE, DROP, INSERT und UPDATE
    * benutzen.
    * 
    * @param expression
    *           SQL-Ausdruck
    */
   private static void update(String expression)
   {
      if (dataBase == null)
         throw new IllegalAccessError(
               "Bitte erst initDataBase() ausfhren");

      try
      {
         dataBase.update(expression);
      }
      catch (SQLException e)
      {
         System.out.println("Fehler in folgendem Statement:");
         System.out.println(expression);
         e.printStackTrace();
      }
   }

   /**
    * Schliet alle Verbindungen und beendet die Datenbank.
    */
   public static void shutdown()
   {
      System.out.println("Datenbank-Shutdown");      
      try
      {
         dataBase.shutdown();
      }
      catch (SQLException e)
      {
         System.out.println("Fehler beim Beenden der Datenbank:");
         e.printStackTrace();
      }
   }
}
