package de.fhswf;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.WindowConstants;

import de.fhswf.Tables.*;

/**
 * 
 * Die Hauptfenster-Klasse.
 * 
 * @author $Author: Thomas Kramer
 * @version $Revision: $, $Date: $ UTC
 */
@SuppressWarnings("serial")
public class MainFrame extends JFrame
{
   JScrollPane viewScrollPane = null;

   public JButton driver = null;

   public JButton cars = null;

   public JButton associations = null;

   final Fahrer driverTable = new Fahrer();

   final Fahrzeug carsTable = new Fahrzeug();

   final Zustaendigkeit associationsTable = new Zustaendigkeit();

   /**
    * Konstruktor.
    */
   public MainFrame()
   {
      super("Aufgabe 8 fr Software-Engineering");

      // Make sure we have nice windows decorations.
      JFrame.setDefaultLookAndFeelDecorated(true);
      addWindowListener(new securityClosing());

      setLayout(new BorderLayout());
      final Container contentPane = getContentPane();
      contentPane.add(new MenuBar(this), BorderLayout.NORTH);

      final JPanel buttonPanel = new JPanel();
      buttonPanel.setLayout(new GridLayout(0, 3));
      driver = new JButton("Fahrer");
      cars = new JButton("Fahrzeuge");
      associations = new JButton("Zuordnungen");

      driver.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(ActionEvent arg0)
         {
            if (viewScrollPane != null)
            {
               contentPane.remove(viewScrollPane);
            }
            viewScrollPane = new JScrollPane(driverTable);
            contentPane.add(viewScrollPane, BorderLayout.CENTER);
            driverTable.clearTable();
            driverTable.selectDrivers();

            // Aufruf der validate-Methode fr Neuzeichnen notwendig!
            contentPane.validate();
            contentPane.repaint();
         }
      });

      cars.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(ActionEvent arg0)
         {
            if (viewScrollPane != null)
            {
               contentPane.remove(viewScrollPane);
            }
            viewScrollPane = new JScrollPane(carsTable);
            contentPane.add(viewScrollPane, BorderLayout.CENTER);
            carsTable.clearTable();
            carsTable.getCars();

            // Aufruf der validate-Methode fr Neuzeichnen notwendig!
            contentPane.validate();
            contentPane.repaint();
         }
      });

      associations.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(ActionEvent arg0)
         {
            if (viewScrollPane != null)
            {
               contentPane.remove(viewScrollPane);
            }
            viewScrollPane = new JScrollPane(associationsTable);
            contentPane.add(viewScrollPane, BorderLayout.CENTER);
            associationsTable.clearTable();
            associationsTable.selectAssociations();

            // Aufruf der validate-Methode fr Neuzeichnen notwendig!
            contentPane.validate();
            contentPane.repaint();
         }
      });

      /**
       * Buttons registrieren.
       */
      buttonPanel.add(driver, BorderLayout.WEST);
      buttonPanel.add(cars, BorderLayout.WEST);
      buttonPanel.add(associations, BorderLayout.WEST);
      contentPane.add(buttonPanel, BorderLayout.SOUTH);

      /**
       * Vorselektion.
       */
      driver.doClick();
   }

   /**
    * 
    * private innere Klasse, um Schlieen ber X-Button optional abzufangen
    * und auf jeden Fall die Datenbank ber DataBaseHelper.shutdown() wieder
    * herunterzufahren.
    * 
    * Den Shutdown-Prozess vor dem Schlieen automatisch aufzurufen (bei
    * Benutzung des X-Buttons) funktionierte nicht wenn die MainFrame-Klasse
    * direkt den WindowsListener implementiert - das windowClosing-Event dort
    * wurde dann nicht aufgerufen obwohl es implementiert war.
    * 
    * So funktioniert es. Man beachte dass die Ausgabe "Datenbank-Shutdown" in
    * jedem Fall erfolgt, egal wie die Anwendung geschlossen wird.
    * 
    * @author $Author: Thomas Kramer
    * @version $Revision: $, $Date: $ UTC
    */
   private class securityClosing extends WindowAdapter
   {
      public void windowClosing(WindowEvent e)
      {
         int option = JOptionPane.showOptionDialog(MainFrame.this,
               "Wollen Sie das Programm wirklich verlassen?",
               "Sicherheitsabfrage", JOptionPane.YES_NO_OPTION,
               JOptionPane.WARNING_MESSAGE, null, null, null);

         if (option == JOptionPane.YES_OPTION)
         {
            DataBaseHelper.shutdown();
            System.exit(0);
         }
         else
         {
            // Abbruch
            setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
         }
      }
   }
}